@@ -20,17 +20,12 @@ use B::Concise qw(set_style);
 
 use if !@ISA, Devel::Trepan::CmdProcessor::Command ;
 
-unless (@ISA) {
-    eval <<"EOE";
-    use constant ALIASES    => qw(disasm);
-    use constant CATEGORY   => 'data';
-    use constant SHORT_HELP => 'Disassemble subroutine(s)';
-    use constant MIN_ARGS  => 0;  # Need at least this many
-    use constant MAX_ARGS  => undef;  # Need at most this many - undef -> unlimited.
-    use constant NEED_STACK => 0;
-
-EOE
-}
+use constant ALIASES    => qw(disasm);
+use constant CATEGORY   => 'data';
+use constant SHORT_HELP => 'Disassemble subroutine(s)';
+use constant MIN_ARGS  => 0;  # Need at least this many
+use constant MAX_ARGS  => undef;  # Need at most this many - undef -> unlimited.
+use constant NEED_STACK => 0;
 
 # OPf_ flags from Perl's op.h
 # Note weird bit combiniation 3, 'want list', we have to handle separately
@@ -69,7 +64,7 @@ use vars qw(@ISA $DEFAULT_OPTIONS);
 use vars @CMD_VARS;  # Value inherited from parent
 
 $DEFAULT_OPTIONS = {
-    line_style => 'debug',
+    line_style => 'terse',
     order      => '-basic',
     tree_style => '-ascii',
 };
@@ -145,7 +140,8 @@ sub parse_options($$)
 	'-highlight'  => sub { $opts->{highlight} = 1; },
 	'-no-highlight' => sub { $opts->{highlight} = 0; },
 	'from=i'     => \$opts->{from},
-	'to=i'       => \$opts->{to}
+	'to=i'       => \$opts->{to},
+	#'addr=s'     => \$opts->{addr},
 	);
     $opts;
 }
@@ -234,10 +230,12 @@ sub markup_basic($$$$$)
     return join("\n", @newlines);
 }
 
-sub markup_tree($$$)
+sub markup_tree($$$$$)
 {
-    my ($lines, $highlight, $proc) = @_;
+    my ($lines, $highlight, $proc, $from, $to) = @_;
     my @lines = split /\n/, $lines;
+    my $current_line = 0;
+    my @newlines = ();
     # use Enbugger 'trepan'; Enbugger->stop;
     my $addr = $proc->{frame}{addr};
     my $filename = $proc->{frame}{file};
@@ -293,14 +291,18 @@ sub markup_tree($$$)
 	    }
 	}
 	$_ = $marker . $_;
+	next if $current_line < $from or $current_line > $to;
+	push @newlines, $_;
     }
-    return join("\n", @lines);
+    return join("\n", @newlines);
 }
 
-sub markup_tree_terse($$$)
+sub markup_tree_terse($$$$$)
 {
-    my ($lines, $highlight, $proc) = @_;
+    my ($lines, $highlight, $proc, $from, $to) = @_;
     my @lines = split /\n/, $lines;
+    my $current_line = 0;
+    my @newlines = ();
     # use Enbugger 'trepan'; Enbugger->stop;
     my $addr = $proc->{frame}{addr};
     my $check_hex_str;
@@ -312,6 +314,7 @@ sub markup_tree_terse($$$)
     	my $marker = '    ';
     	if (/^(\s*)# (\d+):(.+)$/) {
     	    my ($space, $lineno, $perl_code) = ($1, $2, $3, $4);
+	    $current_line = $lineno;
     	    my $marked = $perl_code;
     	    # FIXME: DRY code with markup_basic
     	    if ($perl_code =~
@@ -358,8 +361,11 @@ sub markup_tree_terse($$$)
     	}
 
     	$_ = $marker . $_;
+	next if $current_line < $from or $current_line > $to;
+	push @newlines, $_;
+
     }
-    return join("\n", @lines);
+    return join("\n", @newlines);
 }
 
 sub do_one($$$$)
@@ -374,9 +380,11 @@ sub do_one($$$$)
     my $highlight = $options->{highlight} && $proc->{settings}{highlight};
     ## FIXME: syntax highlight the output.
     if ('terse' eq $options->{line_style}) {
-	$buf = markup_tree_terse($buf, $options->{highlight}, $proc);
+	$buf = markup_tree_terse($buf, $options->{highlight}, $proc,
+				 $options->{from}, $options->{to});
     } elsif ('-tree' eq $options->{order}) {
-	$buf = markup_tree($buf, $options->{highlight}, $proc);
+	$buf = markup_tree($buf, $options->{highlight}, $proc, $options->{from},
+			   $options->{to});
     } elsif ('-basic' eq $options->{order}) {
 	$buf = markup_basic($buf, $options->{highlight}, $proc, $options->{from},
 			    $options->{to});
@@ -437,10 +445,9 @@ unless (caller) {
     };
     $proc->{settings}{highlight} = 1;
     my $cmd = __PACKAGE__->new($proc);
-    # $cmd->run([$NAME, '-terse', '--highlight']);
-    # print '=' x 50, "\n";
-    $cmd->run([$NAME, '-tree', '--highlight']);
+    $cmd->run([$NAME, '-terse', '--highlight']);
     print '=' x 50, "\n";
+    # print '=' x 50, "\n";
     # $cmd->run([$NAME, '-basic', '--highlight']);
     # print '=' x 50, "\n";
     # $cmd->run([$NAME, '-basic', '--highlight', '-from', 10, '-to',  20]);
